using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using MWDataGrabber.SubRecords;

namespace MWDataGrabber.Records
{
    public class Cell : Record
    {
        private bool _interior;
        private bool _hasWater;
        private bool _sleepingIllegal;
        private bool _behaveLikeExterior;
        private int _gridX;
        private int _gridY;
        private Location _minXY = new Location( int.MaxValue, int.MaxValue );
        private Location _maxXY = new Location( int.MinValue, int.MinValue );
        private string _region;
        private List<CellObject> _objects = new List<CellObject>();
        private CellObject _currentObject = null;

        public Cell( BinaryReader source )
            : base( source )
        {
			_type = RecordType.Cell;
        }

        private Door RecastAsDoor()
        {
            Door door = _currentObject as Door;
            if ( door == null )
            {
                door = Door.DoorFromObject( _currentObject );
                _objects.Remove( _currentObject );
                _currentObject = door;
                _objects.Add( door );
            }
            return door;
        }

        protected override void ProcessSubRecord( string id, int length )
        {
            switch ( id )
            {
                case "NAME":
                    if ( _currentObject == null )
                    {
                        _id = ReadString( length );
                    }
                    else
                    {
                        _currentObject.Name = ReadString( length );
                    }
                    break;
                case "DATA":
                    if ( _currentObject == null )
                    {
                        int tmp = ReadInt();
                        _interior = ( ( tmp & 1 ) == 1 );
                        _hasWater = ( ( tmp & 2 ) == 2 );
                        _sleepingIllegal = ( ( tmp & 4 ) == 4 );
                        _behaveLikeExterior = ( ( tmp & 128 ) == 128 );
                        _gridX = ReadInt();
                        _gridY = ReadInt();
                    }
                    else
                    {
                        _currentObject.X = ReadFloat();
                        _currentObject.Y = ReadFloat();
                        _currentObject.Z = ReadFloat();
                        _currentObject.XRotation = ReadFloat();
                        _currentObject.YRotation = ReadFloat();
                        _currentObject.ZRotation = ReadFloat();
                        if ( _currentObject.X < _minXY.X )
                            _minXY.X = (int)_currentObject.X;
                        if ( _currentObject.Y < _minXY.Y )
                            _minXY.Y = (int)_currentObject.Y;
                        if ( _currentObject.X > _maxXY.X )
                            _maxXY.X = (int)_currentObject.X;
                        if ( _currentObject.Y > _maxXY.Y )
                            _maxXY.Y = (int)_currentObject.Y;
                    }
                    break;
                case "RGNN":
                    _region = ReadString( length );
                    break;
                case "FRMR":
                    _currentObject = new CellObject( ReadInt() );
                    _objects.Add( _currentObject );
                    break;
                case "DODT":
                    {
                        Door door = RecastAsDoor();
                        door.DoorX = ReadFloat();
                        door.DoorY = ReadFloat();
                        door.DoorZ = ReadFloat();
                        door.DoorRotationX = ReadFloat();
                        door.DoorRotationY = ReadFloat();
                        door.DoorRotationZ = ReadFloat();
                        break;
                    }
                case "DNAM":
                    RecastAsDoor().Name = ReadString( length );
                    break;
                case "KNAM":
                    _currentObject.KeyName = ReadString( length );
                    break;
                case "FLTV":
                    _currentObject.LockLevel = ReadInt();
                    break;
                default:
                    base.ProcessSubRecord( id, length );
                    break;
            }

        }

        public List<CellObject> Objects
        {
            get { return _objects; }
        }

        public string Region
        {
            get { return _region; }
        }

        public bool Interior
        {
            get { return _interior; }
        }

		public bool City
		{
			get
			{
				return !_interior && !string.IsNullOrEmpty( _id );
			}
		}

        public override string Id
        {
            get
            {
                if ( string.IsNullOrEmpty( _id ) )
                {
                    return string.Format( "{0} ({1},{2})", _region, _gridX, _gridY );
                }
                else
                {
                    return _id;
                }
            }
        }

		public int GridX
		{
			get
			{
				return _gridX;
			}
		}

		public int GridY
		{

			get
			{
				return _gridY;
			}
		}

        public Location MinXY { get { return _minXY; } }
        public Location MaxXY { get { return _maxXY; } }
    }
}